program LiczbyErdosa ;

{ programowanie : Przemyslaw Broniek }

const
  TEXTBUFFER_SIZE  = 100000 ;

  MAX_AUTHORS 	   = 100000 ;
  MAX_QUESTIONS    = 100000 ;
  MAX_PUBLICATIONS = 100000 ;
  MAX_AUTHORNAME   =     10 ; 

  HASH_SIZE        = 110017 ;

type
  Integer = Longint ;
  TQuestion = record Year : Integer ; Author : Integer ; Answer : Integer end ;
  PList = ^TList ;
  TPublication = record Year : Integer ; AuthorsList : PList ; end ;
  TList = record Author : Integer ; Next : PList ; end ;

var
  Tasks : Integer ;
  TextBuffer1 : array [ 1..TEXTBUFFER_SIZE ] of Byte ;
  TextBuffer2 : array [ 1..TEXTBUFFER_SIZE ] of Byte ;
  
  AuthorsCount : Integer ;
  AuthorsHash : array [ 0..HASH_SIZE - 1 ] of Integer ;
  AuthorsNames : array [ 1..MAX_AUTHORS ] of String[ MAX_AUTHORNAME ] ;

  ErdosFlag : Boolean ;
  ErdosNo : array [ 0..MAX_AUTHORS, Boolean ] of Integer ;

  Questions : array [ 1..MAX_QUESTIONS ] of TQuestion ;
  QuestionsOrder : array [ 1..MAX_QUESTIONS ] of Integer ;
  QuestionsCount : Integer ;
  
  Publications : array [ 1..MAX_PUBLICATIONS ] of TPublication ;
  PublicationsOrder : array [ 1..MAX_PUBLICATIONS ] of Integer ;
  PublicationsCount : Integer ;

//*** HASH ***

function Hash( Name : String ) : Integer ;
var
  I : Integer ;
  Res : Longword ;
begin
  Res := 0 ;
  for I := 1 to Length( Name ) do
    Res := Ord( Name[ I ] ) + Res * 65599 ; 
  Hash := Res mod HASH_SIZE ;
end ;

procedure HashInitialize ;
begin
  FillChar( AuthorsHash, SizeOf( AuthorsHash ), 0 ) ;
  AuthorsCount := 0 ;
end ;

function HashFind( Name : String ) : Integer ;
var
  I : Integer ;
begin
  I := Hash( Name ) ;
  while AuthorsHash[ I ] <> 0 do
  begin
    if AuthorsNames[ AuthorsHash[ I ] ] = Name then break ;
    Inc( I ) ;
    if I = HASH_SIZE then I := 0 ;
  end ;
  HashFind := I ;
end ;

function HashInsert( Name : String ) : Integer ;
var
  I : Integer ;
begin
  I := HashFind( Name ) ;
  if AuthorsHash[ I ] = 0 then
  begin
    Inc( AuthorsCount ) ;
    AuthorsHash[ I ] := AuthorsCount ;
    AuthorsNames[ AuthorsCount ] := Name ;
  end ;
  HashInsert := AuthorsHash[ I ] ; 
end ;

function FindAuthor( Name : String ) : Integer ;
begin
  if Name = 'Erdos' then
    FindAuthor := 0
  else
    FindAuthor := HashInsert( Name ) ;
end ;

//*** INPUT ***

procedure ReadPublications ;
var
  I, J, CJ : Integer ;
  Nr : Integer ;
  S, Name : String ;
  P : PList ;
begin
  for I := 1 to PublicationsCount do
  begin
    ReadLn( S ) ;
    Publications[ I ].Year := 
      ( Ord( S[ 1 ] ) - Ord( '0' ) ) * 1000 + 
      ( Ord( S[ 2 ] ) - Ord( '0' ) ) * 100 + 
      ( Ord( S[ 3 ] ) - Ord( '0' ) ) * 10 + 
        Ord( S[ 4 ] ) - Ord( '0' ) ;
    Publications[ I ].AuthorsList := nil ;
    J := 6 ;
    while J <= Length( S ) do
    begin
      CJ := J ;
      repeat
        Inc( CJ ) ;
      until ( CJ > Length( S ) ) or ( S[ CJ ] = ' ' ) ;
      Name := Copy( S, J, CJ - J ) ;      
      J := CJ + 1 ;
      Nr := FindAuthor( Name ) ;
      New( P ) ;
      P^.Next := Publications[ I ].AuthorsList ;
      P^.Author := Nr ;
      Publications[ I ].AuthorsList := P ;
    end ;  
  end ;
end ;

procedure ReadQuestions ;
var
  I : Integer ;
  Name : String ;
begin
  for I := 1 to QuestionsCount do
  begin
    ReadLn( Name ) ;
    Questions[ I ].Year := 
      ( Ord( Name[ 1 ] ) - Ord( '0' ) ) * 1000 + 
      ( Ord( Name[ 2 ] ) - Ord( '0' ) ) * 100 + 
      ( Ord( Name[ 3 ] ) - Ord( '0' ) ) * 10 + 
        Ord( Name[ 4 ] ) - Ord( '0' ) ;
    Name := Copy( Name, 6, Length( Name ) - 5 ) ;
    Questions[ I ].Answer := -1 ;
    Questions[ I ].Author := FindAuthor( Name ) ;
  end ;
end ;

//*** SOLUTION ***

procedure SortData ;
var
  I : Integer ;
  Pos : array [ 1913..2005 ] of Integer ;
begin
  for I := 1913 to 2005 do
    Pos[ I ] := 0 ;
  for I := 1 to QuestionsCount do
    Inc( Pos[ Questions[ I ].Year ] ) ;
  for I := 1914 to 2005 do
    Pos[ I ] := Pos[ I ] + Pos[ I - 1 ] ;
  for I := 1 to QuestionsCount do
  begin
    QuestionsOrder[ Pos[ Questions[ I ].Year ] ] := I ;
    Dec( Pos[ Questions[ I ].Year ] ) ;
  end ;
  
  for I := 1913 to 2005 do
    Pos[ I ] := 0 ;
  for I := 1 to PublicationsCount do
    Inc( Pos[ Publications[ I ].Year ] ) ;
  for I := 1914 to 2005 do
    Pos[ I ] := Pos[ I ] + Pos[ I - 1 ] ;
  for I := 1 to PublicationsCount do
  begin
    PublicationsOrder[ Pos[ Publications[ I ].Year ] ] := I ;
    Dec( Pos[ Publications[ I ].Year ] ) ;
  end ;
end ;

procedure AnswerQuestions ;
var
  I : Integer ;
begin
  for I := 1 to QuestionsCount do
    if Questions[ I ].Answer = -1 then
      WriteLn( 'NIESKONCZONA' )
    else
      WriteLn( Questions[ I ].Answer ) ;
end ;

procedure ComputeAnswers ;
var
  I, Publicated, PublicatedYear : Integer ;
  PQ : ^TQuestion ;

  procedure PublicateYear( Year : Integer ) ;
  var
    P : TPublication ;

    procedure NewYear ;
    var
      I : Integer ;
    begin
      ErdosFlag := not ErdosFlag ;
      for I := 0 to AuthorsCount do
        ErdosNo[ I, ErdosFlag ] :=
	  ErdosNo[ I, not ErdosFlag ] ; 
    end ;

    procedure Publicate( P : TPublication ) ;
    var
      S, SDis : PList ; 
      CE, MinE : Integer ;
    begin
      if PublicatedYear < P.Year then
      begin
        NewYear ;
	PublicatedYear := P.Year ;
      end ;
      Inc( Publicated ) ;
      S := P.AuthorsList ;
      MinE := -1 ;
      while S <> nil do
      begin
        CE := ErdosNo[ S^.Author, not ErdosFlag ] ;
        if CE <> -1 then
  	begin
 	  if MinE = -1 then
	    MinE := CE
	  else if CE < MinE then
	    MinE := CE ;	    
	end ;
	S := S^.Next ;
      end ;
      if MinE <> -1 then
      begin
        S := P.AuthorsList ;
        while S <> nil do
  	begin
 	  CE := ErdosNo[ S^.Author, ErdosFlag ] ;
	  if ( CE = -1 ) or ( CE > MinE + 1 ) then
	    CE := MinE + 1 ;
 	  ErdosNo[ S^.Author, ErdosFlag ] := CE ;
	  S := S^.Next ;
	end ;
      end ;
      S := P.AuthorsList ;
      while S <> nil do
      begin
	SDis := S ;
	S := S^.Next ;
	Dispose( SDis ) ;
      end ;
    end ;
    
  begin
    while ( Publicated < PublicationsCount ) do
    begin
      P := Publications[ PublicationsOrder[ Publicated + 1 ] ] ;
      if P.Year <= Year then 
        Publicate( P ) 
      else
        break ;
    end ;
  end ;
  
begin
  ErdosFlag := False ;
  for I := 1 to AuthorsCount do
    ErdosNo[ I, ErdosFlag ] := -1 ;
  ErdosNo[ 0, ErdosFlag ] := 0 ;
  
  Publicated := 0 ;
  PublicatedYear := 1912 ;
  for I := 1 to QuestionsCount do
  begin
    PQ := @Questions[ QuestionsOrder[ I ] ] ;
    PublicateYear( PQ^.Year ) ;
    PQ^.Answer := ErdosNo[ PQ^.Author, ErdosFlag ] ;
  end ;
end ;

begin
  SetTextBuf( Input, TextBuffer1, TEXTBUFFER_SIZE ) ;
  SetTextBuf( Output, TextBuffer2, TEXTBUFFER_SIZE ) ;
  ReadLn( Tasks ) ;
  while Tasks > 0 do
  begin
    Dec( Tasks ) ;
    HashInitialize ;
    ReadLn( PublicationsCount, QuestionsCount ) ;
    ReadPublications ;    
    ReadQuestions ;
    SortData ;
    ComputeAnswers ;
    AnswerQuestions ;
  end ;
end.

